/* styling for all html tags */
body{
  background: #ccc;
  color: grey;
  border-color: grey;
}
table{
  font-size: 0.9em;
}
pre{
  overflow: visible !important;
}

/* styling for generic classes */
.bold{
  font-weight: bold;
}
.pre{
  white-space: pre;
}
.monospace{
  font-family: Menlo,Monaco,Consolas,"Courier New",monospace;
}
.memory_address{
  color: #337ab7;
  text-decoration: none;
  font-weight:
}
.lighttext{
  color: #4a4a4a;
}
.glyphicon{
  /* glyphicons are too bold, make them lighter*/
    color: #848484;
}
.flex{
  display: flex;
}
.flexrow{
  display: flex;
  flex-direction: row;
}
/* components get their own titlebars */
.titlebar{
  width: 100%;
  color: black;
  background-color: white;
  height: 25px;
}
.pointer{
  cursor: pointer;
}
.gdb_content_div{
  overflow: auto;
  height: 150px;
  background-color: #f7f7f7;
  border-color: grey;
  border-style: solid;
  border-width: 1px;
  border-radius: 2px;
}
.sent_command:hover{
  /* lighten background */
  background-color:rgba(255,255,255,0.1)
}
.disabled {
    z-index: 1000;
    background-color: lightgrey;
    opacity: 0.6;
    pointer-events: none;
}
.margin_sm{
  margin: 2px;
}
.no_padding{
  padding: 2px !important;
}
.code pre, table.code {
    margin: 0px;
    padding: 0px;
    border-width: 0px;
    width: 100%;
    background-color: transparent;
}
.highlight{
  background: rgba(255, 255, 0, 0.5) !important;
}
.line_num{
  width: 50px;
  border-width: 0;
  border-right: 1px;
  border-style: solid;
  border-color: #c7c7c7;
  padding-left: 10px;
  padding-right: 10px;
  font-size: 0.9em;
  color: #ababab;
  cursor: pointer;
}
/* the line number has its style changed if it has a breakpoint */
.line_num.breakpoint{
  background: #33cdff !important;
  color: black;
  border-style: solid;
  border-color: #000000;
  border-width: 1px;
}
td.assembly{
  /*background-color: white;*/
  padding-bottom: 6px;
/*  min-width: 100px;
  max-width: 200px;*/
  /*overflow: auto;*/
  white-space: nowrap;
  /*word-wrap: break-word;*/
}
#code_table{
  font-family: Menlo,Monaco,Consolas,"Courier New",monospace;
}
/* set background color over souce code only */
.source_code_row.line_of_code td{
  background-color: #f7f7f7;
  max-width: 80%;
}
.line_of_code pre{
  margin-left: 10px;
}
/* when hovering, set background color of entire row */
.source_code_row:hover td, .source_code_row:hover td pre{
  background-color: #e2e2e2;
}
.padding_left{
  padding-left: 5px;
}
.line_of_code{
  height: 18px
}
.dropdown-btn {
    vertical-align: top;
    height: 30px;
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
}
/* show a flash of color */
.flash {
    -webkit-animation-name: flash-animation;
    -webkit-animation-duration: 1.0s;

    animation-name: flash-animation;
    animation-duration: 1.0s;
}

@-webkit-keyframes flash-animation {
    from { background: yellow; }
    to   { background: default; }
}

@keyframes flash-animation {
    from { background: yellow; }
    to   { background: default; }
}

/* specific styling for ids or components */
#always_on_top{
  position: fixed;
  top: 0;
  margin-top: 0;
  height: 102px;
  width: 100%;
  border-bottom: black;
  border-style: solid;
  border-width: 0px;
  border-bottom-width: 1px;
  z-index: 1000;
  background: #f1f1f1;
  border-bottom-color: #ccc;
}
#console{
  font-size: 0.9em !important;
  background-color: #292929 !important;
  color: #f9f9f9 !important;
  font-family: Menlo,Monaco,Consolas,"Courier New",monospace !important;
}
#gdb_command{
  padding-left: 45px;
  border-top: 0;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
  font-size: 0.9em !important;
  background-color: #3c3c3c !important;
  color: #f9f9f9 !important;
  font-family: Menlo,Monaco,Consolas,"Courier New",monospace !important;
}
#gdb_mi_output{
  font-family: Menlo,Monaco,Consolas,"Courier New",monospace;
  overflow: auto;
  font-size: 0.9em;
}

/* styling for "Variables" component */
#variables li{
  list-style: none;
}

#threads ul{
  list-style: none;
  padding: 0;
}


#variables ul.variable{
    padding-left: 5px;
    border: 0px;
}
/* Show type of variable in lighter text */
.var_type{
  color: #a0a0a0;
  margin-left: 5px;
  font-style: italic;
}

/* Make refresh icon spin - http://www.bootply.com/128062*/
.glyphicon-refresh-animate {
    -animation: spin .7s infinite linear;
    -webkit-animation: spin2 .7s infinite linear;
}

@-webkit-keyframes spin2 {
    from { -webkit-transform: rotate(0deg);}
    to { -webkit-transform: rotate(360deg);}
}

@keyframes spin {
    from { transform: scale(1) rotate(0deg);}
    to { transform: scale(1) rotate(360deg);}
}


footer a{
  color: #ccc;
}
footer ul{
  list-style: none;
  padding: 0;
}

/* Vendor Overrides */

/* awesomeplete wraps inputs in a div, and the div is really small. Make it big. */
div.awesomplete{
  width: 100%;
}
/* auto-complete library (awesomplete) has unlimited dropdown size.
Limit to 200px high, and add scrollbar, so it doesn't take over the page */
.awesomplete  ul {
  overflow: auto;
  max-height: 200px;
}
